==> nui_core-0.0.127.tar.gz: lib/bloc/nui_bloc.dart <==
import 'dart:async';
import 'package:flutter/cupertino.dart';

abstract class NUIBloc<T> {
  StreamController<T> controller = StreamController();
  Stream<T> get stream => controller.stream;
  Sink<T> get sink => controller.sink;
  bool _isDisposed = false;

  void notify(T data){
    if(_isDisposed) return;
    sink.add(data);
  }

  void dispose() {
    _isDisposed = true;
    controller.close();
  }
}

class NUIBlocData<T> extends NUIBloc<T>{}

enum BlocStatus{
  LOADING,
  FAILED,
  COMPLETED
}

typedef BlocStreamInitialData<T> = T Function(BuildContext context);
typedef BlocStreamFutureData<T> = Widget Function(BuildContext context, BlocStatus status, T snapshot);

class NUIBlocStream<T> extends StatefulWidget {
  NUIBlocStream({@required this.bloc, this.initial, @required this.stream, this.autoDispose: true});
  final NUIBloc<T> bloc;
  final T initial;
  final BlocStreamFutureData<T> stream;
  final bool autoDispose;
  @override
  _NUIBlocStreamState<T> createState() => _NUIBlocStreamState<T>();
}

class _NUIBlocStreamState<T> extends State<NUIBlocStream<T>> {
  @override
  Widget build(BuildContext context) {
    return StreamBuilder<T>(
      initialData: widget.initial,
      stream: widget.bloc.stream,
      builder: (BuildContext context, AsyncSnapshot<T> snapshot) {
        if(snapshot.hasData){
          return widget.stream(context, BlocStatus.COMPLETED, snapshot.data);
        }
        if(snapshot.hasError){
          return widget.stream(context, BlocStatus.FAILED, null);
        }
        else{
          return widget.stream(context, BlocStatus.LOADING, null);
        }
      },
    );
  }

  @override
  void dispose(){
    super.dispose();
    if(widget.autoDispose == true) {
      widget.bloc.dispose();
    }
  }
}

class NUIBlocWidget<T extends NUIBloc> extends StatefulWidget {
  final Widget child;
  final T bloc;

  const NUIBlocWidget({Key key, @required this.bloc, @required this.child}) : super(key: key);

  @override
  State createState() => _NUIBlocWidgetState();
}

class _NUIBlocWidgetState extends State<NUIBlocWidget> {
  @override
  Widget build(BuildContext context) => widget.child;

  @override
  void dispose() {
    widget.bloc.dispose();
    super.dispose();
  }
}

==> nui_core-1.0.4.tar.gz: lib/bloc/nui_bloc.dart <==
import 'dart:async';
import 'package:flutter/cupertino.dart';

abstract class NUIBloc<T> {
  StreamController<T> controller = StreamController();
  Stream<T?> get stream => controller.stream;
  Sink<T?> get sink => controller.sink;
  bool _isDisposed = false;

  void notify(T data){
    if(_isDisposed) return;
    sink.add(data);
  }

  void dispose() {
    _isDisposed = true;
    controller.close();
  }
}

class NUIBlocData<T> extends NUIBloc<T>{}

enum BlocStatus{
  LOADING,
  FAILED,
  COMPLETED
}

typedef BlocStreamInitialData<T> = T? Function(BuildContext context);
typedef BlocStreamFutureData<T> = Widget Function(BuildContext context, BlocStatus status, T? snapshot);

class NUIBlocStream<T> extends StatefulWidget {
  NUIBlocStream({required this.bloc, this.initial, required this.stream, this.autoDispose= true});
  final NUIBloc<T> bloc;
  final T? initial;
  final BlocStreamFutureData<T> stream;
  final bool autoDispose;
  @override
  _NUIBlocStreamState<T> createState() => _NUIBlocStreamState<T>();
}

class _NUIBlocStreamState<T> extends State<NUIBlocStream<T>> {
  @override
  Widget build(BuildContext context) {
    return StreamBuilder<T?>(
      initialData: widget.initial,
      stream: widget.bloc.stream,
      builder: (BuildContext context, AsyncSnapshot<T?> snapshot) {
        if(snapshot.hasData){
          return widget.stream(context, BlocStatus.COMPLETED, snapshot.data);
        }
        if(snapshot.hasError){
          return widget.stream(context, BlocStatus.FAILED, null);
        }
        else{
          return widget.stream(context, BlocStatus.LOADING, null);
        }
      },
    );
  }

  @override
  void dispose(){
    super.dispose();
    if(widget.autoDispose == true) {
      widget.bloc.dispose();
    }
  }
}

class NUIBlocWidget<T extends NUIBloc> extends StatefulWidget {
  final Widget child;
  final T bloc;

  const NUIBlocWidget({Key? key, required this.bloc, required this.child}) : super(key: key);

  @override
  State createState() => _NUIBlocWidgetState();
}

class _NUIBlocWidgetState extends State<NUIBlocWidget> {
  @override
  Widget build(BuildContext context) => widget.child;

  @override
  void dispose() {
    widget.bloc.dispose();
    super.dispose();
  }
}
